Class: Net::HTTPResponse

Inherits:
Object
  • Object
show all
Includes:
HTTPHeader
Defined in:
lib/mechanize/net-overrides/net/http.rb,
lib/mechanize/net-overrides/net/http.rb

Overview

redefine

Constant Summary collapse

CODE_CLASS_TO_OBJ =
{
  '1' => HTTPInformation,
  '2' => HTTPSuccess,
  '3' => HTTPRedirection,
  '4' => HTTPClientError,
  '5' => HTTPServerError
}
CODE_TO_OBJ =
{
  '100' => HTTPContinue,
  '101' => HTTPSwitchProtocol,

  '200' => HTTPOK,
  '201' => HTTPCreated,
  '202' => HTTPAccepted,
  '203' => HTTPNonAuthoritativeInformation,
  '204' => HTTPNoContent,
  '205' => HTTPResetContent,
  '206' => HTTPPartialContent,

  '300' => HTTPMultipleChoice,
  '301' => HTTPMovedPermanently,
  '302' => HTTPFound,
  '303' => HTTPSeeOther,
  '304' => HTTPNotModified,
  '305' => HTTPUseProxy,
  '307' => HTTPTemporaryRedirect,

  '400' => HTTPBadRequest,
  '401' => HTTPUnauthorized,
  '402' => HTTPPaymentRequired,
  '403' => HTTPForbidden,
  '404' => HTTPNotFound,
  '405' => HTTPMethodNotAllowed,
  '406' => HTTPNotAcceptable,
  '407' => HTTPProxyAuthenticationRequired,
  '408' => HTTPRequestTimeOut,
  '409' => HTTPConflict,
  '410' => HTTPGone,
  '411' => HTTPLengthRequired,
  '412' => HTTPPreconditionFailed,
  '413' => HTTPRequestEntityTooLarge,
  '414' => HTTPRequestURITooLong,
  '415' => HTTPUnsupportedMediaType,
  '416' => HTTPRequestedRangeNotSatisfiable,
  '417' => HTTPExpectationFailed,

  '501' => HTTPInternalServerError,
  '501' => HTTPNotImplemented,
  '502' => HTTPBadGateway,
  '503' => HTTPServiceUnavailable,
  '504' => HTTPGatewayTimeOut,
  '505' => HTTPVersionNotSupported
}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from HTTPHeader

#[], #[]=, #add_field, #basic_auth, #chunked?, #content_length, #content_length=, #content_range, #content_type, #delete, #each_capitalized, #each_capitalized_name, #each_header, #each_name, #each_value, #fetch, #get_fields, #initialize_http_header, #key?, #main_type, #proxy_basic_auth, #range, #range_length, #set_content_type, #set_range, #size, #sub_type, #to_hash, #type_params

Constructor Details

#initialize(httpv, code, msg) ⇒ HTTPResponse

:nodoc: internal use only



1879
1880
1881
1882
1883
1884
1885
1886
# File 'lib/mechanize/net-overrides/net/http.rb', line 1879

def initialize(httpv, code, msg)   #:nodoc: internal use only
  @http_version = httpv
  @code         = code
  @message      = msg
  initialize_http_header nil
  @body = nil
  @read = false
end

Instance Attribute Details

#codeObject (readonly)

HTTP result code string. For example, ‘302’. You can also determine the response type by which response subclass the response object is an instance of.



1894
1895
1896
# File 'lib/mechanize/net-overrides/net/http.rb', line 1894

def code
  @code
end

#http_versionObject (readonly)

The HTTP version supported by the server.



1889
1890
1891
# File 'lib/mechanize/net-overrides/net/http.rb', line 1889

def http_version
  @http_version
end

#messageObject (readonly) Also known as: msg

HTTP result message. For example, ‘Not Found’.



1897
1898
1899
# File 'lib/mechanize/net-overrides/net/http.rb', line 1897

def message
  @message
end

Class Method Details

.body_permitted?Boolean

true if the response has body.

Returns:

  • (Boolean)


1614
1615
1616
# File 'lib/mechanize/net-overrides/net/http.rb', line 1614

def HTTPResponse.body_permitted?
  self::HAS_BODY
end

.exception_typeObject

:nodoc: internal use only



1618
1619
1620
# File 'lib/mechanize/net-overrides/net/http.rb', line 1618

def HTTPResponse.exception_type   # :nodoc: internal use only
  self::EXCEPTION_TYPE
end

.read_new(sock) ⇒ Object

:nodoc: internal use only



1838
1839
1840
1841
1842
1843
1844
1845
# File 'lib/mechanize/net-overrides/net/http.rb', line 1838

def read_new(sock)   #:nodoc: internal use only
  httpv, code, msg = read_status_line(sock)
  res = response_class(code).new(httpv, code, msg)
  each_response_header(sock) do |k,v|
    res.add_field k, v
  end
  res
end

Instance Method Details

#bodyObject Also known as: entity

Returns the entity body.

Calling this method a second or subsequent time will return the already read string.

http.request_get('/index.html') {|res|
  puts res.body
}

http.request_get('/index.html') {|res|
  p res.body.object_id   # 538149362
  p res.body.object_id   # 538149362
}


2027
2028
2029
# File 'lib/mechanize/net-overrides/net/http.rb', line 2027

def body
  read_body()
end

#code_typeObject

response <-> exception relationship



1922
1923
1924
# File 'lib/mechanize/net-overrides/net/http.rb', line 1922

def code_type   #:nodoc:
  self.class
end

#error!Object

:nodoc:

Raises:



1926
1927
1928
# File 'lib/mechanize/net-overrides/net/http.rb', line 1926

def error!   #:nodoc:
  raise error_type().new(@code + ' ' + @message.dump, self)
end

#error_typeObject

:nodoc:



1930
1931
1932
# File 'lib/mechanize/net-overrides/net/http.rb', line 1930

def error_type   #:nodoc:
  self.class::EXCEPTION_TYPE
end

#headerObject

:nodoc:



1948
1949
1950
1951
# File 'lib/mechanize/net-overrides/net/http.rb', line 1948

def header   #:nodoc:
  warn "#{caller(1)[0]}: warning: HTTPResponse#header is obsolete" if $VERBOSE
  self
end

#inspectObject



1900
1901
1902
# File 'lib/mechanize/net-overrides/net/http.rb', line 1900

def inspect
  "#<#{self.class} #{@code} #{@message} readbody=#{@read}>"
end

#read_body(dest = nil, &block) ⇒ Object

Gets entity body. If the block given, yields it to block. The body is provided in fragments, as it is read in from the socket.

Calling this method a second or subsequent time will return the already read string.

http.request_get('/index.html') {|res|
  puts res.read_body
}

http.request_get('/index.html') {|res|
  p res.read_body.object_id   # 538149362
  p res.read_body.object_id   # 538149362
}

# using iterator
http.request_get('/index.html') {|res|
  res.read_body do |segment|
    print segment
  end
}


1995
1996
1997
1998
1999
2000
2001
2002
2003
2004
2005
2006
2007
2008
2009
2010
2011
# File 'lib/mechanize/net-overrides/net/http.rb', line 1995

def read_body(dest = nil, &block)
  if @read
    raise IOError, "#{self.class}\#read_body called twice" if dest or block
    return @body
  end
  to = procdest(dest, block)
  stream_check
  if @body_exist
    read_body_0 to
    @body = to
  else
    @body = nil
  end
  @read = true

  @body
end

#read_headerObject

:nodoc:



1953
1954
1955
1956
# File 'lib/mechanize/net-overrides/net/http.rb', line 1953

def read_header   #:nodoc:
  warn "#{caller(1)[0]}: warning: HTTPResponse#read_header is obsolete" if $VERBOSE
  self
end

#reading_body(sock, reqmethodallowbody) ⇒ Object

body



1962
1963
1964
1965
1966
1967
1968
1969
1970
1971
# File 'lib/mechanize/net-overrides/net/http.rb', line 1962

def reading_body(sock, reqmethodallowbody)  #:nodoc: internal use only
  @socket = sock
  @body_exist = reqmethodallowbody && self.class.body_permitted?
  begin
    yield
    self.body   # ensure to read body
  ensure
    @socket = nil
  end
end

#responseObject

header (for backward compatibility only; DO NOT USE)



1943
1944
1945
1946
# File 'lib/mechanize/net-overrides/net/http.rb', line 1943

def response   #:nodoc:
  warn "#{caller(1)[0]}: warning: HTTPResponse#response is obsolete" if $VERBOSE
  self
end

#to_aryObject

For backward compatibility. To allow Net::HTTP 1.1 style assignment e.g.

response, body = Net::HTTP.get(....)


1909
1910
1911
1912
1913
1914
1915
1916
# File 'lib/mechanize/net-overrides/net/http.rb', line 1909

def to_ary
  warn "net/http.rb: warning: Net::HTTP v1.1 style assignment found at #{caller(1)[0]}; use `response = http.get(...)' instead." if $VERBOSE
  res = self.dup
  class << res
    undef to_ary
  end
  [res, res.body]
end

#valueObject

Raises HTTP error if the response is not 2xx.



1935
1936
1937
# File 'lib/mechanize/net-overrides/net/http.rb', line 1935

def value
  error! unless self.kind_of?(HTTPSuccess)
end