Method: Mechanize::HTTP::Agent#post_connect_hooks
- Defined in:
- lib/mechanize/http/agent.rb
#post_connect_hooks ⇒ Object (readonly)
A list of hooks to call after retrieving a response. Hooks are called with the agent and the response returned.
39 40 41 |
# File 'lib/mechanize/http/agent.rb', line 39 def post_connect_hooks @post_connect_hooks end |